//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "DetectorConstruction.hh"
#include "SensitiveDetector.hh"

#include "G4RunManager.hh"

#include "G4Material.hh"
#include "G4NistManager.hh"

#include "G4ExtrudedSolid.hh"
#include "G4Polyhedra.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4SubtractionSolid.hh"

#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4SystemOfUnits.hh"
#include "G4PhysicalConstants.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4PhysicalConstants.hh"
#include "G4SystemOfUnits.hh"
#include "globals.hh"

#include "G4VSensitiveDetector.hh"
#include "G4MultiFunctionalDetector.hh"
#include "G4SDManager.hh"
#include "G4PSDoseDeposit.hh"
#include "G4PSFlatSurfaceFlux.hh"
#include "G4PSEnergyDeposit.hh"
#include "G4SDParticleFilter.hh"
#include "G4VSDFilter.hh"
#include "G4SDKineticEnergyFilter.hh"

#include <G4UniformMagField.hh>
#include <G4FieldManager.hh>
#include <G4TransportationManager.hh>
#include <G4ChordFinder.hh>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorConstruction::DetectorConstruction()
{
	G4cout << "### DetectorConstruction instantiated ###" << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

DetectorConstruction::~DetectorConstruction() {}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void DetectorConstruction::DefineMaterials()
{ 
	//variables
	G4String name, symbol;
	G4int Z;
	G4double A;	
	G4double density = 1.29*mg/cm3;
	G4int nc = 2;
	
	//air
	G4Element* elN = new G4Element(name="Nitrogen", symbol="N", Z=7, A=14.01*g/mole);
	G4Element* elO = new G4Element("Oxygen", "O", 8, 16.*g/mole);
	Air = new G4Material("air", density, nc);
	Air->AddElement(elN, 0.7);
	Air->AddElement(elO, 0.3);
	
	//water
	Z = 1;
	A = 1.01*g/mole;
    density = 1.*g/cm3;
	G4Element* elH = new G4Element("Hydrogen", "H", Z, A);
	Water = new G4Material("water", density, nc);
	Water->AddElement(elH, 2);
	Water->AddElement(elO, 1);
	
	//Define materials from the NIST database
	//Lead = G4NistManager::Instance()->FindOrBuildMaterial("G4_Pb");
  	G4NistManager* NistMan = G4NistManager::Instance();
  	Lead = NistMan->FindOrBuildMaterial("G4_Pb"); 
  	Vacuum = NistMan->FindOrBuildMaterial("G4_Galactic");


  	//Print all the materials defined
  	G4cout << *(G4Material::GetMaterialTable()) << G4endl;
    G4cout << "###############################" << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4VPhysicalVolume* DetectorConstruction::Construct(){
    
	//call defined materials
    DefineMaterials();

    //variable for enabling the geometry check
    G4bool checkOverlaps = true;
       

	//initialization of geometrical variables
	fa = 300*mm;
	fb = 300*mm;
	fc = 0.01*mm;
    
   
    //World    
    G4double worldSize = 10.*CLHEP::meter;
    
    G4Box* worldSolid = new G4Box("World",
                                  worldSize/2.,
                                  worldSize/2.,
                                  worldSize/2.);
    
    G4LogicalVolume* worldLogic = new G4LogicalVolume(worldSolid,
                                  	                  Vacuum,
                                                      "World");

	worldLogic->SetVisAttributes(G4VisAttributes::GetInvisible());
	  
    G4PVPlacement* worldPhysical = new G4PVPlacement(0,
                                                     G4ThreeVector(),
                                                     worldLogic,
                                                     "World",
                                                     0,
                                                     false,
                                                     0,
													 checkOverlaps);


	//tringular prism
    std::vector<G4TwoVector> poligon;
    poligon.push_back(G4TwoVector(-10*cm, 0));
    poligon.push_back(G4TwoVector(0, 25*cm));
    poligon.push_back(G4TwoVector(10*cm, 0));
	G4double hz = 5*cm;
	G4TwoVector offset1 = G4TwoVector(0,0);
  	G4TwoVector offset2 = G4TwoVector(0,0);
	G4double scale1 = 1.;
	G4double scale2 = 1.;
	G4ExtrudedSolid* myPolyHedra = new G4ExtrudedSolid("myPolyhedraSolid",
												 		poligon,
														hz,
														offset1, 
														scale1,
												    	offset2,
														scale2);
												
	G4VisAttributes* PolyhedraVisAttribute = new G4VisAttributes(G4Colour(1., 1., 0.));  
	PolyhedraVisAttribute->SetForceSolid(true);
	
	G4RotationMatrix* myRotationMatrix = new G4RotationMatrix();
	myRotationMatrix->rotateX(90.*CLHEP::deg);
	
	G4ThreeVector myTranslationVector;
												
/*	
    myPolyhedraLogical = new G4LogicalVolume(myPolyHedra,
                                          	 Water,
                                       		 "myPolyhedraLogical");
                                       		 
    myPolyhedraLogical2 = new G4LogicalVolume(myPolyHedra,
                                          	  Water,
                                       		  "myPolyhedraLogical2");

	myPolyhedraLogical->SetVisAttributes(PolyhedraVisAttribute);
	myPolyhedraLogical2->SetVisAttributes(PolyhedraVisAttribute);
*/

	G4String scoringVolumeName;
	std::stringstream istream;
	G4double scoringVolumeX[fNumberOfScoringVolumes] = {-10.*cm, 10.*cm};

	for (int i = 0; i < fNumberOfScoringVolumes; i++) {
	    istream << i;
		scoringVolumeName = "myPolyhedraLogical" + istream.str();
		
		fScoringVolume[i] = new G4LogicalVolume(myPolyHedra,
                                          	    Water,
                                       		    scoringVolumeName);
                                       		    
        fScoringVolume[i]->SetVisAttributes(PolyhedraVisAttribute);
                                       		      
		new G4PVPlacement(myRotationMatrix,
		                  G4ThreeVector(scoringVolumeX[i], 0.*cm, 40.*cm),
		                  fScoringVolume[i],
		                  "myPolyhedraPhysical",
		                  worldLogic,
		                  false,
		                  i,
						  checkOverlaps);                                         		    
	}
												
	
	//virtual (scoring screen) SensitiveDetector										
	G4Box* myBox = new G4Box("mySolidVolume",
							 fa*0.5,
							 fa*0.5,
							 fc*0.5);
						
	myLogicalVolume = new G4LogicalVolume(myBox,
	                                      Vacuum,
	                                      "myLogicalVolume");
	                                    
	new G4PVPlacement(0,
                      G4ThreeVector(0.*cm, 0.*cm, 10.*cm),
                      myLogicalVolume,
                      "myPhysicallVolume",
                      worldLogic,
                      false,
                      0,
					  checkOverlaps);
								  
			
	//solid to contain a constant magnetic field		  
	G4Tubs* MFsolid = new G4Tubs("MFsolid",
                                 0., 
                         		 12.*cm, 
                         		 10.*cm, 
                         		 0.*deg, 
                                 360.*deg);  
	
	G4LogicalVolume* MFlogic = new G4LogicalVolume(MFsolid,
	                                               Vacuum,
	                                               "MFlogic");
	                                               
	G4VisAttributes* MFVisAttribute = new G4VisAttributes(G4Colour(1., 0., 0., 0.3));  
	MFVisAttribute->SetForceSolid(true);
	MFlogic->SetVisAttributes(MFVisAttribute);
	                                               
	G4RotationMatrix* MFrotation = new G4RotationMatrix();
	MFrotation->rotateX(90.*CLHEP::deg);
	                                    
	new G4PVPlacement(MFrotation,
                      G4ThreeVector(0.*cm, 0.*cm, -5.*cm),
                      MFlogic,
                      "MFphysical",
                      worldLogic,
                      false,
                      0,
					  checkOverlaps);					  
					                                   
						
    //always return the world physical volume			
    return worldPhysical;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void DetectorConstruction::ConstructSDandField()
{
	//Sensitive Detector
	G4VSensitiveDetector* vDetectror = new SensitiveDetector("det");
	G4SDManager::GetSDMpointer()->AddNewDetector(vDetectror);
	myLogicalVolume->SetSensitiveDetector(vDetectror); //use this if I want a true virtual scoring
	//myPolyhedraLogical->SetSensitiveDetector(vDetectror);
	//myPolyhedraLogical2->SetSensitiveDetector(vDetectror);
	//for (int i = 0; i < fNumberOfScoringVolumes; i++)
	//	fScoringVolume[i]->SetSensitiveDetector(vDetectror);
	
	//built-in Edep scorer
    G4MultiFunctionalDetector* multisd = new G4MultiFunctionalDetector("multisd");
    G4VPrimitiveScorer* EdepDet = new G4PSEnergyDeposit("edep");
    multisd->RegisterPrimitive(EdepDet);
    G4SDManager::GetSDMpointer()->AddNewDetector(multisd);
    //SetSensitiveDetector(myPolyhedraLogical->GetName(), multisd);
    //SetSensitiveDetector(myPolyhedraLogical2->GetName(), multisd);
    for (int i = 0; i < fNumberOfScoringVolumes; i++)
		SetSensitiveDetector(fScoringVolume[i]->GetName(), multisd);
    	
	
	//Magnetic Field
	if (fSetMagneticField) {
		G4UniformMagField* myField = 
			new G4UniformMagField(G4ThreeVector(0., 1.*tesla, 0.));
		G4FieldManager* localfieldMgr = new G4FieldManager(myField);
		localfieldMgr->CreateChordFinder(myField);
		G4LogicalVolume* fieldLog = 
			G4LogicalVolumeStore::GetInstance()->GetVolume("MFlogic");
		fieldLog->SetFieldManager(localfieldMgr, true);
	} 
	
	G4cout << "### End of DetectorConstruction ###" << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo..

