//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "PrimaryGeneratorAction.hh"
#include "DetectorConstruction.hh"

#include "G4RunManager.hh"
#include "G4Event.hh"
#include "G4AutoLock.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"

#include "Randomize.hh"

#include "G4GeneralParticleSource.hh"
#include "G4ParticleGun.hh"

#include "G4PhysicalConstants.hh"
#include "G4SystemOfUnits.hh"
#include "globals.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PrimaryGeneratorAction::PrimaryGeneratorAction()
{
	G4cout << "### PrimaryGeneratorAction instantiated ###" << G4endl;
	
	fMessenger = new PrimaryGeneratorActionMessenger(this);
	
    fGPS = new G4GeneralParticleSource();   
    fGun = new G4ParticleGun(); 
    
    IWantGPS = true;
    
    //default values for GPS (used if no macro are defined)
    //Primary particles
    G4ParticleDefinition* particle = 
    	G4ParticleTable::GetParticleTable()->FindParticle("e-");
    fGPS->SetParticleDefinition(particle);  
    //Spatial distribution
    G4SPSPosDistribution* vPosDist = fGPS->GetCurrentSource()->GetPosDist();
    vPosDist->SetPosDisType("Plane");
    vPosDist->SetPosDisShape("Circle");
    vPosDist->SetRadius(50.*mm);   
    vPosDist->SetCentreCoords(G4ThreeVector(0.,0.,0.));
    //Angular distribution
    G4SPSAngDistribution* vAngDist = fGPS->GetCurrentSource()->GetAngDist();
    vAngDist->SetParticleMomentumDirection(G4ThreeVector(0., 0., 1.));  
    //Energy distribution
    G4SPSEneDistribution* vEneDist = fGPS->GetCurrentSource()->GetEneDist();
    vEneDist->SetEnergyDisType("Mono");
    vEneDist->SetMonoEnergy(400.*keV);
    
    fPrimariesPerEvent = 1;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PrimaryGeneratorAction::~PrimaryGeneratorAction()
{
	delete fGPS;
	delete fGun;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
	if (IWantGPS) {
		
		//fGPS->GeneratePrimaryVertex(anEvent);
		for (int ii = 0; ii < fPrimariesPerEvent; ii++) {
			fGPS->GeneratePrimaryVertex(anEvent);
		}	
		
	} else {	
	
		//option1: randomization
		//particle type
		G4ParticleDefinition* particle = 
			G4ParticleTable::GetParticleTable()->FindParticle("gamma");	
		fGun->SetParticleDefinition(particle);
		
		//number of particles per event
		fGun->SetNumberOfParticles(2);
		
		//Get something from the DetectorConstruction
		const DetectorConstruction* detectorConstruction = 
			static_cast<const DetectorConstruction*>(G4RunManager::GetRunManager()
				->GetUserDetectorConstruction());		
		//detectorConstruction->GetSomething(); //I can retrieve what I need...
		G4double a = detectorConstruction->GetA();
		G4double b = detectorConstruction->GetB();
		G4double c = detectorConstruction->GetC();
		
		//particle postion
		//G4double a = 10.*cm;
		//G4double b = 10.*cm;
		//G4double c = 10.*cm;
		G4double x = -a*0.5 + a*G4UniformRand();
		G4double y = -b*0.5 + b*G4UniformRand();
		//G4double z = -c*0.5 + c*G4UniformRand();
		G4double z = -40*cm;
		fGun->SetParticlePosition(G4ThreeVector(x,y,z));
		
		//particle direction
		G4double u = 0.;
		G4double v = 0.;
		G4double w = 1.;

		//isotropical source
		/*
		G4double cosTheta = -1. + 2.*G4UniformRand();
		G4double phi = G4UniformRand()*twopi;
		G4double sinTheta = sqrt(1. - cosTheta*cosTheta);
		u = sinTheta*sin(phi);
		v = sinTheta*cos(phi);
		w = cosTheta;
		*/
		G4ThreeVector direction(u,v,w);		
		fGun->SetParticleMomentumDirection(direction);	
			
		//particle energy
		G4double E = G4RandGauss::shoot(59.*keV,0.*keV);
		fGun->SetParticleEnergy(E);
		
		//generate the particle
		fGun->GeneratePrimaryVertex(anEvent);
				
/*		
		//option2: 2 vertices
		G4ParticleDefinition* particle = 
			G4ParticleTable::GetParticleTable()->FindParticle("gamma");
		fGun->SetParticleDefinition(particle);
		fGun->SetParticlePosition(G4ThreeVector(0.,0.,-40*cm));
		fGun->SetParticleEnergy(100.*keV);
		fGun->SetParticleMomentumDirection(G4ThreeVector(0.4,0.,1.));
		fGun->GeneratePrimaryVertex(anEvent);
		
		G4ParticleDefinition* particle2 = 
			G4ParticleTable::GetParticleTable()->FindParticle("e+");
		fGun->SetParticleDefinition(particle2);
		fGun->SetParticlePosition(G4ThreeVector(0.,0.,40*cm));
		fGun->SetParticleEnergy(1.*MeV);
		fGun->SetParticleMomentumDirection(G4ThreeVector(-0.4,0.,1.));
		fGun->GeneratePrimaryVertex(anEvent);
*/
		
	}
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

